/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
   Copyright (C) 2006 Chris Lord <chris@openedhand.com>
   
   derived from dom-styleevent.c,
   
   Copyright (C) 2000 CodeFactory AB
   Copyright (C) 2000 Jonas Borgstr\366m <jonas@codefactory.se>
   Copyright (C) 2000 Anders Carlsson <andersca@codefactory.se>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "dom-attrevent.h"

static DomEventClass *parent_class = NULL;

const DomString *
dom_AttrEvent__get_attrName (DomAttrEvent *event)
{
	return event->attrName;
}

gushort
dom_AttrEvent__get_attrChange (DomAttrEvent *event)
{
	return event->attrChange;
}

const DomString *
dom_AttrEvent__get_prevValue (DomAttrEvent *event)
{
	return event->prevValue;
}

const DomString *
dom_AttrEvent__get_newValue (DomAttrEvent *event)
{
	return event->newValue;
}

DomNode *
dom_AttrEvent__get_relatedNode (DomAttrEvent *event)
{
	return event->relatedNode;
}

void
dom_AttrEvent_initAttrEvent (DomAttrEvent *event, const DomString *typeArg, DomBoolean canBubbleArg, DomBoolean cancelableArg, const DomString *attrNameArg, gushort attrChangeArg, const DomString *prevValueArg, const DomString *newValueArg, DomNode *relatedNodeArg)
{
	dom_Event_initEvent (DOM_EVENT (event), typeArg, canBubbleArg, cancelableArg);

	if (event->attrName)
		g_free (event->attrName);

	if (event->prevValue)
		g_free (event->prevValue);

	if (event->newValue)
		g_free (event->newValue);

	if (event->relatedNode)
		g_object_unref (event->relatedNode);

	event->attrName = g_strdup (attrNameArg);
	event->attrChange = attrChangeArg;
	event->prevValue = g_strdup (prevValueArg);
	event->newValue = g_strdup (newValueArg);
	event->relatedNode = g_object_ref (relatedNodeArg);
}

static void
dom_attr_event_finalize (GObject *object)
{
	DomAttrEvent *event = DOM_ATTR_EVENT (object);
	
	if (event->attrName)
		g_free (event->attrName);

	if (event->prevValue)
		g_free (event->prevValue);

	if (event->newValue)
		g_free (event->newValue);

	if (event->relatedNode)
		g_object_unref (event->relatedNode);
		
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
dom_attr_event_class_init (DomAttrEventClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;

	object_class->finalize = dom_attr_event_finalize;

	parent_class = g_type_class_peek_parent (klass);
}

GType
dom_attr_event_get_type (void)
{
	static GType dom_attr_event_type = 0;

	if (!dom_attr_event_type) {
		static const GTypeInfo dom_attr_event_info = {
			sizeof (DomAttrEventClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) dom_attr_event_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (DomAttrEvent),
			16,   /* n_preallocs */
			NULL,
		};

		dom_attr_event_type = g_type_register_static (DOM_TYPE_EVENT, "DomAttrEvent", &dom_attr_event_info, 0);
	}

	return dom_attr_event_type;
}

