/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
   Copyright (C) 2000 CodeFactory AB
   Copyright (C) 2000 Jonas Borgström <jonas@codefactory.se>
   Copyright (C) 2000 Anders Carlsson <andersca@codefactory.se>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>

#include <libgtkhtml/gtkhtmlcontext.h>
#include "browser-window.h"
#include "glibcurl.h"
#include "url-utils.h"

void usage(char *self) {
	self = basename(strdup(self));
	g_print("\
%s (C) libgtkhtml authors 2000, James B 2015.\n\
Usage: %s [--debug] [--insecure] [--no-pp] [initial-url] [home-url]\n\
  --debug     : debug network calls.\n\
  --insecure  : enable untrusted certs when using SSL.\n\
  --no-pp     : disable HTTP pipelining\n\
  --online    : allow network connections\n\
  initial-url : the initial URL you want to open.\n\
  home-url    : the URL of the \"home page\", if different from initial url\n\n\
", 
	self, self);
	exit(0);
}

gint
main (gint argc, gchar **argv)
{
	GtkWidget *browser;
	gchar *initial_url = NULL;
	gchar *home_url = NULL;
	gboolean debug = FALSE;
	gboolean insecure = FALSE;
	gboolean pipelining = TRUE;
	gboolean online = FALSE;
	
	gchar *self = argv[0];
	while (*++argv) {
		if (strcmp(*argv,"--debug")==0)
			debug = TRUE;
		else if (strcmp(*argv,"--insecure")==0)
			insecure = TRUE;
		else if (strcmp(*argv,"--no-pp")==0)
			pipelining = FALSE;
		else if (strcmp(*argv,"--online")==0)
			online = TRUE;
		else if (strcmp(*argv,"--help")==0 || strcmp(*argv,"-h")==0)
			usage(self);
		else if (initial_url==NULL)
			initial_url = *argv;
		else if (home_url==NULL)
			home_url = *argv;
	}

	g_thread_init(NULL);
	gtk_init (&argc, &argv);
	glibcurl_init();

	browser = browser_window_new (NULL);
	browser_window_set_title (browser, "GtkHtml II MAIN");
	g_signal_connect (G_OBJECT (browser), "destroy",
			  G_CALLBACK (gtk_main_quit), NULL);

	if (pipelining) {
		curl_multi_setopt(glibcurl_handle(), CURLMOPT_PIPELINING, 1); 
	} else g_print("Pipelining disabled\n");

	if (debug) browser_window_set_debug (browser, TRUE);
	if (insecure) browser_window_set_insecure (browser, TRUE);
	if (online) browser_window_set_online (browser, TRUE);
	if (initial_url) {
		browser_window_set_home_url(browser, initial_url);
		browser_window_load_url(browser, initial_url);
	}
	if (home_url) browser_window_set_home_url(browser, home_url);
	
	gtk_widget_show (browser);
	gtk_main ();

	return 0;
}
